from dynaconf import Dynaconf

CONFIG = Dynaconf(
    envvar_prefix="AKB",
    settings_files=["config.yaml", ".env"],
    env_switcher="ENV",
    core_loaders=["YAML"],
    environments=True,
    load_dotenv=True,
)



import uuid
import datetime

from gaea.models import (
    Users,
    Credentials,
    Owners,
    Swarms,
    SwarmHealthStatuses,
    Hives,
    HiveConditions,
    Apiaries,
    HoneyTypes,
    Comments,
    Events,
    EventStatuses,
    EventTypes,
)


def generate_uuid(nb):
    return tuple(uuid.uuid4() for _ in range(nb))


USER_EMAIL = "coucou@cou.com"
USER_USERNAME = "coucou"
USER_PASSWORD = "password"
SWARM_HEALTH_STATUS_GOOD = "Good"
SWARM_HEALTH_STATUS_BAD = "Bad"
HIVES_CONDITION_NEW = "New"
HIVES_CONDITION_OLD = "Old"
COMMENT_TYPE_USER = "User"
COMMENT_TYPE_SYSTEM = "System"
EVENT_STATUS_PLANIFIED = "Planified"
EVENT_STATUS_DONE = "Done"
EVENT_TYPE_BURN_BEES = "Burn them all"
EVENT_TYPE_WATER_BEEHOUSE = "Water it"
HONEY_TYPE_CHESTNUT = "Chestnut"
HONEY_TYPE_ALL_FLOWERS = "All flowers"

IDS = {
    "Users": generate_uuid(2),
    "Credentials": generate_uuid(2),
    "Owners": generate_uuid(3),
    "Swarms": generate_uuid(4),
    "Hives": generate_uuid(4),
    "Apiaries": generate_uuid(4),
    "Swarm_health_statuses": generate_uuid(3),
    "Honey_types": generate_uuid(3),
    "Hive_conditions": generate_uuid(3),
    "Comment_types": generate_uuid(3),
    "Event_types": generate_uuid(3),
    "Event_statuses": generate_uuid(3),
    "Events": generate_uuid(3),
}

DATASETS = (
    (
        Users(id=IDS["Users"][0], email=USER_EMAIL),
        Users(id=IDS["Users"][-1], email="unknown_user@gmail.com"),
    ),
    (
        Credentials(
            id=IDS["Credentials"][0],
            username=USER_USERNAME,
            password=b"password",
            user_id=IDS["Users"][0],
        ),
        Credentials(
            id=IDS["Credentials"][-1],
            username="unknown_user",
            password=b"password",
            user_id=IDS["Users"][-1],
        ),
        Owners(id=IDS["Owners"][0], name="owner1", user_id=IDS["Users"][0]),
        Owners(id=IDS["Owners"][1], name="owner2", user_id=IDS["Users"][0]),
        Owners(id=IDS["Owners"][-1], name="unknown_owner", user_id=IDS["Users"][-1]),
        SwarmHealthStatuses(
            id=IDS["Swarm_health_statuses"][0],
            name=SWARM_HEALTH_STATUS_GOOD,
            user_id=IDS["Users"][0],
        ),
        SwarmHealthStatuses(
            id=IDS["Swarm_health_statuses"][1],
            name=SWARM_HEALTH_STATUS_BAD,
            user_id=IDS["Users"][0],
        ),
        SwarmHealthStatuses(
            id=IDS["Swarm_health_statuses"][-1],
            name="unknown",
            user_id=IDS["Users"][-1],
        ),
        HoneyTypes(
            id=IDS["Honey_types"][0],
            name=HONEY_TYPE_ALL_FLOWERS,
            user_id=IDS["Users"][0],
        ),
        HoneyTypes(
            id=IDS["Honey_types"][1], name=HONEY_TYPE_CHESTNUT, user_id=IDS["Users"][0]
        ),
        HoneyTypes(id=IDS["Honey_types"][-1], name="unknown", user_id=IDS["Users"][-1]),
        HiveConditions(
            id=IDS["Hive_conditions"][0],
            name=HIVES_CONDITION_NEW,
            user_id=IDS["Users"][0],
        ),
        HiveConditions(
            id=IDS["Hive_conditions"][1],
            name=HIVES_CONDITION_OLD,
            user_id=IDS["Users"][0],
        ),
        HiveConditions(
            id=IDS["Hive_conditions"][-1], name="unknown", user_id=IDS["Users"][-1]
        ),
        EventTypes(
            id=IDS["Event_types"][0], name=EVENT_TYPE_BURN_BEES, user_id=IDS["Users"][0]
        ),
        EventTypes(
            id=IDS["Event_types"][1],
            name=EVENT_TYPE_WATER_BEEHOUSE,
            user_id=IDS["Users"][0],
        ),
        EventTypes(
            id=IDS["Event_types"][-1],
            name="unknown",
            user_id=IDS["Users"][-1],
        ),
        EventStatuses(
            id=IDS["Event_statuses"][0],
            name=EVENT_STATUS_PLANIFIED,
            user_id=IDS["Users"][0],
        ),
        EventStatuses(
            id=IDS["Event_statuses"][1], name=EVENT_STATUS_DONE, user_id=IDS["Users"][0]
        ),
        EventStatuses(
            id=IDS["Event_statuses"][-1], name="unknown", user_id=IDS["Users"][-1]
        ),
    ),
    (
        Swarms(
            id=IDS["Swarms"][0],
            health_status_id=IDS["Swarm_health_statuses"][0],
            queen_year=2020,
            user_id=IDS["Users"][0],
        ),
        Swarms(
            id=IDS["Swarms"][1],
            health_status_id=IDS["Swarm_health_statuses"][0],
            queen_year=2021,
            user_id=IDS["Users"][0],
        ),
        Swarms(
            id=IDS["Swarms"][2],
            health_status_id=IDS["Swarm_health_statuses"][1],
            queen_year=2018,
            user_id=IDS["Users"][0],
        ),
        Swarms(
            id=IDS["Swarms"][-1],
            health_status_id=IDS["Swarm_health_statuses"][-1],
            queen_year=2019,
            user_id=IDS["Users"][-1],
        ),
        Apiaries(
            id=IDS["Apiaries"][0],
            name="apiary1",
            location="location1",
            user_id=IDS["Users"][0],
            honey_type_id=IDS["Honey_types"][0],
        ),
        Apiaries(
            id=IDS["Apiaries"][1],
            name="apiary2",
            location="location2",
            user_id=IDS["Users"][0],
            honey_type_id=IDS["Honey_types"][1],
        ),
        Apiaries(
            id=IDS["Apiaries"][2],
            name="apiary3",
            location="location3",
            user_id=IDS["Users"][0],
            honey_type_id=IDS["Honey_types"][1],
        ),
        Apiaries(
            id=IDS["Apiaries"][-1],
            name="unknown",
            location="unknown",
            user_id=IDS["Users"][-1],
            honey_type_id=IDS["Honey_types"][-1],
        ),
        Hives(
            id=IDS["Hives"][0],
            name="hive1",
            user_id=IDS["Users"][0],
            condition_id=IDS["Hive_conditions"][0],
            owner_id=IDS["Owners"][0],
            swarm_id=IDS["Swarms"][0],
            apiary_id=IDS["Apiaries"][0],
        ),
        Hives(
            id=IDS["Hives"][1],
            name="hive2",
            user_id=IDS["Users"][0],
            condition_id=IDS["Hive_conditions"][0],
            owner_id=IDS["Owners"][1],
            swarm_id=IDS["Swarms"][1],
            apiary_id=IDS["Apiaries"][1],
        ),
        Hives(
            id=IDS["Hives"][2],
            name="hive3",
            user_id=IDS["Users"][0],
            condition_id=IDS["Hive_conditions"][1],
            owner_id=IDS["Owners"][0],
            swarm_id=IDS["Swarms"][2],
            apiary_id=IDS["Apiaries"][2],
        ),
        Hives(
            id=IDS["Hives"][-1],
            name="unknown",
            user_id=IDS["Users"][-1],
            condition_id=IDS["Hive_conditions"][-1],
            owner_id=IDS["Owners"][-1],
            swarm_id=IDS["Swarms"][-1],
            apiary_id=IDS["Apiaries"][-1],
        ),
        Events(
            id=IDS["Events"][0],
            title="title1",
            description="description1",
            due_date=datetime.datetime.utcnow(),
            user_id=IDS["Users"][0],
            type_id=IDS["Event_types"][0],
            status_id=IDS["Event_statuses"][1],
            hive_id=IDS["Hives"][0],
        ),
        Events(
            id=IDS["Events"][1],
            title="title2",
            description="description2",
            due_date=datetime.datetime.utcnow() + datetime.timedelta(days=1),
            user_id=IDS["Users"][0],
            type_id=IDS["Event_types"][1],
            status_id=IDS["Event_statuses"][0],
            hive_id=IDS["Hives"][0],
        ),
        Events(
            id=IDS["Events"][-1],
            title="unknown",
            description="unknown",
            due_date=datetime.datetime.utcnow() + datetime.timedelta(days=1),
            user_id=IDS["Users"][-1],
            type_id=IDS["Event_types"][-1],
            status_id=IDS["Event_statuses"][-1],
            hive_id=IDS["Hives"][-1],
        ),
        Comments(
            id=uuid.uuid4(),
            comment="comment1",
            type="user",
            date=datetime.datetime.utcnow(),
            user_id=IDS["Users"][0],
            swarm_id=IDS["Swarms"][0],
            hive_id=IDS["Hives"][0],
        ),
        Comments(
            id=uuid.uuid4(),
            comment="comment2",
            type="user",
            date=datetime.datetime.utcnow(),
            user_id=IDS["Users"][0],
            event_id=IDS["Events"][0],
        ),
        Comments(
            id=uuid.uuid4(),
            comment="comment3",
            type="user",
            date=datetime.datetime.utcnow(),
            user_id=IDS["Users"][0],
            swarm_id=IDS["Swarms"][1],
            hive_id=IDS["Hives"][1],
        ),
        Comments(
            id=uuid.uuid4(),
            comment="comment4",
            type="user",
            date=datetime.datetime.utcnow(),
            user_id=IDS["Users"][0],
            swarm_id=IDS["Swarms"][0],
            hive_id=IDS["Hives"][0],
        ),
        Comments(
            id=uuid.uuid4(),
            comment="unknown",
            type="user",
            date=datetime.datetime.utcnow(),
            user_id=IDS["Users"][-1],
            swarm_id=IDS["Swarms"][-1],
            hive_id=IDS["Hives"][-1],
        ),
    ),
)



import pika

from gaea.log import logger

from .base import RBMQClient


class RBMQConsumer(RBMQClient):
    def __init__(
        self,
        handlers,
        queue,
        exchange=None,
        connection_manager=None,
        dead_letter_exchange=None,
    ):
        super().__init__(
            exchange=exchange,
            connection_manager=connection_manager,
            dead_letter_exchange=dead_letter_exchange,
        )
        self.handlers = handlers
        self.queue = queue

    def setup(self, channel):
        self.declare_queue(channel=channel)
        self.bind_queues(channel=channel)

    def declare_queue(self, channel):
        channel.queue_declare(
            queue=self.queue,
            durable=True,
            arguments={"x-dead-letter-exchange": self.dead_letter_exchange},
        )
        channel.queue_declare(queue=f"{self.queue}-dead-letter", durable=True)

    def bind_queues(self, channel):
        for routing_key in self.handlers:
            channel.queue_bind(
                queue=self.queue, exchange=self.exchange, routing_key=routing_key
            )
            channel.queue_bind(
                queue=f"{self.queue}-dead-letter",
                exchange=self.dead_letter_exchange,
                routing_key=routing_key,
            )

    def consume(self):
        while True:
            logger.info("Starting consuming...", queue=self.queue)
            try:
                with self.connection_manager as channel:
                    self.setup(channel)
                    channel.basic_consume(
                        queue=self.queue, on_message_callback=self.process
                    )
                    channel.start_consuming()
            except KeyboardInterrupt:
                logger.info("Goodbye")
                self.connection_manager.close()
                return
            except pika.exceptions.StreamLostError:
                logger.warning(
                    "RBMQ connection stream has been lost. Trying to reconnect"
                )
                continue
            except:
                logger.exception("Critical error on consumer")
                self.connection_manager.close()
                raise

    def process(self, channel, method, properties, body):
        log_details = dict(
            properties=properties, body=body, routing_key=method.routing_key
        )
        logger.info(f"New message received {method.routing_key}", **log_details)

        handler = self.handlers.get(method.routing_key)
        if handler is None:
            logger.warning("Unknown routing key", **log_details)
            channel.basic_nack(method.delivery_tag, requeue=False)
            return

        try:
            success = handler(properties, body)
            if success:
                channel.basic_ack(method.delivery_tag)
            else:
                channel.basic_nack(method.delivery_tag, requeue=False)
        except:  # pylint: disable=bare-except
            logger.exception("Critical error when handling rbmq message", **log_details)
            channel.basic_nack(method.delivery_tag, requeue=False)



import fastapi
from fastapi.middleware.cors import CORSMiddleware

from .monitoring import router as monitoring_router


class AppClient:
    def __init__(self, routers, middleware=None):
        self.app = fastapi.FastAPI()

        if not isinstance(routers, (list, tuple)):
            routers = [routers]

        self.app.include_router(monitoring_router)
        for router in routers:
            self.app.include_router(router)

        self.middleware = middleware

    def _add_middleware(self):
        if self.middleware:
            self.app.middleware("http")(self.middleware)

    def add_cors_middleware(
        self,
        allow_origins=None,
        allow_origin_regex=None,
        allow_methods=None,
        allow_headers=None,
        allow_credentials=False,
    ):
        params = dict(
            allow_methods=allow_methods or ["*"],
            allow_headers=allow_headers or ["*"],
            allow_credentials=allow_credentials,
        )

        if allow_origin_regex:
            params["allow_origin_regex"] = allow_origin_regex
        else:
            params["allow_origins"] = allow_origins or ["*"]

        self.app.add_middleware(CORSMiddleware, **params)

    def get_app(self):
        self._add_middleware()
        return self.app



import pika

from gaea.config import CONFIG
from gaea.errors import AlreadyInitialized
from gaea.log import logger

RBMQ_DEFAULT_EXCHANGE = "akingbee.main_exchange"
RBMQ_DEAD_LETTER_EXCHANGE = "akingbee.dead_letter"


class RBMQClient:
    def __init__(
        self, exchange=None, connection_manager=None, dead_letter_exchange=None
    ):
        self.connection_manager = connection_manager or RBMQConnectionManager()
        self.exchange = exchange or RBMQ_DEFAULT_EXCHANGE
        self.dead_letter_exchange = dead_letter_exchange or RBMQ_DEAD_LETTER_EXCHANGE

        with self.connection_manager as channel:
            self.declare_exchange(channel=channel)

    def declare_exchange(self, channel):
        channel.exchange_declare(
            exchange=self.exchange,
            exchange_type="topic",
            durable=True,
        )
        channel.exchange_declare(
            exchange=RBMQ_DEAD_LETTER_EXCHANGE,
            exchange_type="topic",
            durable=True,
        )

    def close(self):
        self.connection_manager.close()


class RBMQConnectionManager:
    def __init__(self):
        self.connection = self._get_connection()
        self._channel = None

    def __enter__(self):
        if self._channel is not None:
            raise AlreadyInitialized("A rabbitmq channel is already in use")
        self._channel = self.get_channel()
        return self._channel

    def __exit__(self, tp, vl, tb):
        self._clear_channel()

    def get_channel(self):
        return self.connection.channel()

    def _clear_channel(self):
        if self._channel:
            self._channel.close()
            self._channel = None

    def close(self):
        self._clear_channel()
        self.connection.close()

    def _get_connection(self):
        params = self._get_connection_parameters()
        return pika.BlockingConnection(parameters=params)

    @staticmethod
    def _get_connection_parameters():
        creds = pika.PlainCredentials(CONFIG.RBMQ_USER, CONFIG.RBMQ_PASSWORD)
        return pika.ConnectionParameters(
            host=CONFIG.RBMQ_HOST,
            port=CONFIG.RBMQ_PORT,
            virtual_host=CONFIG.RBMQ_VHOST,
            credentials=creds,
            blocked_connection_timeout=300,
        )



import datetime
import uuid

from sqlalchemy import Column, ForeignKey, DDL, event
from sqlalchemy.orm import relationship, column_property
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users


class HoneyTypes(Base):
    __tablename__ = "honey_types"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT())
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="honey_types")


class Apiaries(Base):
    __tablename__ = "apiaries"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    location = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    honey_type_id = Column(
        UUID(as_uuid=True), ForeignKey(HoneyTypes.id), nullable=False
    )

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="apiaries")
    honey_type = relationship(HoneyTypes)

    _nb_hives = None

    @property
    def nb_hives(self):
        if self._nb_hives is None:
            self._nb_hives = len(self.hives)  # pylint: disable=no-member
        return self._nb_hives


func = DDL(
    """
    CREATE FUNCTION check_user_apiaries() RETURNS trigger AS $check_user_apiaries$
        BEGIN
            IF NEW.user_id NOT IN (SELECT t.user_id FROM honey_types as t WHERE t.id = NEW.honey_type_id) THEN
                RAISE EXCEPTION 'Different user for honey_type';
            END IF;

            RETURN NEW;
        END; $check_user_apiaries$ LANGUAGE PLPGSQL
"""
)

trigger = DDL(
    """
    CREATE TRIGGER trigger_user_apiaries BEFORE INSERT OR UPDATE ON apiaries
        FOR EACH ROW EXECUTE PROCEDURE check_user_apiaries();
"""
)

event.listen(Apiaries.metadata, "after_create", func.execute_if(dialect="postgresql"))
event.listen(
    Apiaries.metadata, "after_create", trigger.execute_if(dialect="postgresql")
)



import datetime
import uuid

from sqlalchemy import Column, ForeignKey, DDL, event
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID, INTEGER

from .base import Base
from .users import Users


class SwarmHealthStatuses(Base):
    __tablename__ = "swarm_health_statuses"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT())
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="swarm_healths")


class Swarms(Base):
    __tablename__ = "swarms"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    health_status_id = Column(
        UUID(as_uuid=True), ForeignKey(SwarmHealthStatuses.id), nullable=False
    )
    queen_year = Column(INTEGER(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="swarms_healths")
    health = relationship(SwarmHealthStatuses)


func = DDL(
    """
    CREATE FUNCTION check_user_swarms() RETURNS trigger AS $check_user_swarms$
        BEGIN
            IF NEW.user_id NOT IN (SELECT t.user_id FROM swarm_health_statuses as t WHERE t.id = NEW.health_status_id) THEN
                RAISE EXCEPTION 'Different user for swarm_health_statuses';
            END IF;

            RETURN NEW;
        END; $check_user_swarms$ LANGUAGE PLPGSQL
"""
)

trigger = DDL(
    """
    CREATE TRIGGER trigger_user_swarms BEFORE INSERT OR UPDATE ON swarms
        FOR EACH ROW EXECUTE PROCEDURE check_user_swarms();
"""
)

event.listen(Swarms.metadata, "after_create", func.execute_if(dialect="postgresql"))
event.listen(Swarms.metadata, "after_create", trigger.execute_if(dialect="postgresql"))



from fastapi import APIRouter
from pydantic import BaseModel

router = APIRouter()


class MonitoringModel(BaseModel):
    status: str


@router.get("/_/status", status_code=200, response_model=MonitoringModel)
async def get_apiaries():
    """
    Moniroting route to know if the application is healthy
    """
    return {"status": "OK"}



import datetime
import uuid

from sqlalchemy import Column, ForeignKey, DDL, event
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users
from .apiaries import Apiaries
from .hives import Hives


class EventTypes(Base):
    __tablename__ = "event_types"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="event_types")


class EventStatuses(Base):
    __tablename__ = "event_statuses"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="event_statuses")


class Events(Base):
    __tablename__ = "events"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    title = Column(TEXT(), nullable=False)
    description = Column(TEXT(), nullable=True)
    due_date = Column(TIMESTAMP(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    type_id = Column(UUID(as_uuid=True), ForeignKey(EventTypes.id), nullable=False)
    status_id = Column(UUID(as_uuid=True), ForeignKey(EventStatuses.id), nullable=False)
    hive_id = Column(UUID(as_uuid=True), ForeignKey(Hives.id), nullable=True)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="events")
    type = relationship(EventTypes)
    status = relationship(EventStatuses)
    hive = relationship(Hives, backref="events")


func = DDL(
    """
    CREATE FUNCTION check_user_events() RETURNS trigger AS $check_user_events$
        BEGIN
            IF NEW.user_id NOT IN (SELECT t.user_id FROM event_types as t WHERE t.id = NEW.type_id) THEN
                RAISE EXCEPTION 'Different user for event_types';
            END IF;

            IF NEW.user_id NOT IN (SELECT t.user_id FROM event_statuses as t WHERE t.id = NEW.status_id) THEN
                RAISE EXCEPTION 'Different user for event_statuses';
            END IF;

            IF NEW.hive_id IS NOT NULL AND NEW.user_id NOT IN (SELECT t.user_id FROM hives as t WHERE t.id = NEW.hive_id) THEN
                RAISE EXCEPTION 'Different user for hives';
            END IF;

            RETURN NEW;
        END; $check_user_events$ LANGUAGE PLPGSQL
"""
)

trigger = DDL(
    """
    CREATE TRIGGER trigger_user_events BEFORE INSERT OR UPDATE ON events
        FOR EACH ROW EXECUTE PROCEDURE check_user_events();
"""
)

event.listen(Events.metadata, "after_create", func.execute_if(dialect="postgresql"))
event.listen(Events.metadata, "after_create", trigger.execute_if(dialect="postgresql"))



import uuid

from fastapi import HTTPException
import aiohttp

from gaea.config import CONFIG


async def validate_access_token(access_token):
    CERBES_URL = f"{CONFIG.CERBES_API_ENDPOINT}:{CONFIG.CERBES_API_PORT}"
    url = f"{CERBES_URL}/check"
    if not url.startswith("http"):
        url = f"http://{url}"

    async with aiohttp.ClientSession() as session:
        async with session.get(url, cookies={"access_token": access_token}) as resp:
            return await resp.json() if resp.status == 200 else None


async def get_logged_in_user(access_token):
    if not access_token:
        raise HTTPException(status_code=401)

    data = await validate_access_token(access_token=access_token)

    if data is None:
        raise HTTPException(status_code=401, detail="Invalid access token")

    return uuid.UUID(data["user_id"])



class MockRBMQConnectionManager:
    def __init__(self, mocked_channel):
        self.mocked_channel = mocked_channel

    def get_channel(self):
        return self.mocked_channel

    def close(self):
        pass

    def __enter__(self):
        return self.mocked_channel

    def __exit__(self, tp, vl, tb):
        self.mocked_channel.close()



import datetime
import json
import pika

from gaea.config import CONFIG
from gaea.log import logger

from .base import RBMQClient


class RBMQPublisher(RBMQClient):
    @staticmethod
    def _get_default_headers():
        return {
            "emitter": CONFIG.SERVICE_NAME,
            "created_at": datetime.datetime.utcnow().isoformat(),
        }

    def publish(self, routing_key, content, headers=None):
        if headers is None:
            headers = {}
        headers.update(self._get_default_headers())
        properties = pika.BasicProperties(headers=headers)
        body = json.dumps(content, default=str)

        try:
            with self.connection_manager as channel:
                channel.basic_publish(
                    exchange=self.exchange,
                    routing_key=routing_key,
                    body=body,
                    properties=properties,
                )
        except:
            logger.exception(
                "Fail to publish message in rbmq",
                body=body,
                properties=properties,
                routing_key=routing_key,
            )
            raise



import datetime
import uuid

from sqlalchemy import Column, ForeignKey, DDL, event
from sqlalchemy.orm import relationship, backref
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users, Owners
from .apiaries import Apiaries
from .swarms import Swarms


class HiveConditions(Base):
    __tablename__ = "hive_conditions"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="hive_conditions")


class Hives(Base):
    __tablename__ = "hives"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)

    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    condition_id = Column(
        UUID(as_uuid=True), ForeignKey(HiveConditions.id), nullable=False
    )
    owner_id = Column(UUID(as_uuid=True), ForeignKey(Owners.id), nullable=True)
    swarm_id = Column(UUID(as_uuid=True), ForeignKey(Swarms.id), nullable=True)
    apiary_id = Column(UUID(as_uuid=True), ForeignKey(Apiaries.id), nullable=True)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="hives")
    owner = relationship(Owners, backref="hives")
    swarm = relationship(Swarms, backref=backref("hive", uselist=False))
    apiary = relationship(Apiaries, backref="hives")
    condition = relationship(HiveConditions, backref="hives")


func = DDL(
    """
    CREATE FUNCTION check_user_hives() RETURNS trigger AS $check_user_hives$
        BEGIN
            IF NEW.user_id NOT IN (SELECT t.user_id FROM owners as t WHERE t.id = NEW.owner_id) THEN
                RAISE EXCEPTION 'Different user for owners';
            END IF;

            IF NEW.swarm_id IS NOT NULL AND NEW.user_id NOT IN (SELECT t.user_id FROM swarms as t WHERE t.id = NEW.swarm_id) THEN
                RAISE EXCEPTION 'Different user for swarms';
            END IF;

            IF NEW.apiary_id IS NOT NULL AND NEW.user_id NOT IN (SELECT t.user_id FROM apiaries as t WHERE t.id = NEW.apiary_id) THEN
                RAISE EXCEPTION 'Different user for apiaries';
            END IF;

            IF NEW.user_id NOT IN (SELECT t.user_id FROM hive_conditions as t WHERE t.id = NEW.condition_id) THEN
                RAISE EXCEPTION 'Different user for hive_conditions';
            END IF;

            RETURN NEW;
        END; $check_user_hives$ LANGUAGE PLPGSQL
"""
)

trigger = DDL(
    """
    CREATE TRIGGER trigger_user_hives BEFORE INSERT OR UPDATE ON hives
        FOR EACH ROW EXECUTE PROCEDURE check_user_hives();
"""
)

event.listen(Hives.metadata, "after_create", func.execute_if(dialect="postgresql"))
event.listen(Hives.metadata, "after_create", trigger.execute_if(dialect="postgresql"))



import datetime
import uuid

from sqlalchemy import Column, ForeignKey, DDL, event
from sqlalchemy.orm import relationship, backref
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users
from .swarms import SwarmHealthStatuses, Swarms
from .apiaries import Apiaries
from .hives import Hives, HiveConditions
from .events import Events


class Comments(Base):
    __tablename__ = "comments"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    comment = Column(TEXT(), nullable=False)
    date = Column(TIMESTAMP(), nullable=False)
    type = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    swarm_id = Column(UUID(as_uuid=True), ForeignKey(Swarms.id), nullable=True)
    hive_id = Column(UUID(as_uuid=True), ForeignKey(Hives.id), nullable=True)
    event_id = Column(UUID(as_uuid=True), ForeignKey(Events.id), nullable=True)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="comments")
    swarm = relationship(Swarms, backref="comments")
    hive = relationship(Hives, backref="comments")
    event = relationship(Events, backref=backref("comment", uselist=False))


func = DDL(
    """
    CREATE FUNCTION check_user_comments() RETURNS trigger AS $check_user_comments$
        BEGIN
            IF new.event_id IS NOT NULL AND NEW.user_id NOT IN (SELECT t.user_id FROM events as t WHERE t.id = NEW.event_id) THEN
                RAISE EXCEPTION 'Different user for eventd';
            END IF;

            IF NEW.swarm_id IS NOT NULL AND NEW.user_id NOT IN (SELECT t.user_id FROM swarms as t WHERE t.id = NEW.swarm_id) THEN
                RAISE EXCEPTION 'Different user for swarms';
            END IF;

            IF NEW.hive_id IS NOT NULL AND NEW.user_id NOT IN (SELECT t.user_id FROM hives as t WHERE t.id = NEW.hive_id) THEN
                RAISE EXCEPTION 'Different user for hives';
            END IF;

            RETURN NEW;
        END; $check_user_comments$ LANGUAGE PLPGSQL
"""
)

trigger = DDL(
    """
    CREATE TRIGGER trigger_user_comments BEFORE INSERT OR UPDATE ON comments
        FOR EACH ROW EXECUTE PROCEDURE check_user_comments();
"""
)

event.listen(Comments.metadata, "after_create", func.execute_if(dialect="postgresql"))
event.listen(
    Comments.metadata, "after_create", trigger.execute_if(dialect="postgresql")
)
